import __path from 'node:path';
import { fileURLToPath as __fileURLToPath } from 'node:url';
import { createRequire as __createRequire } from 'node:module';
const require = __createRequire(import.meta.url);
const __filename = __fileURLToPath(import.meta.url);
const __dirname = __path.dirname(__filename);
import {
  HTTP_METHODS,
  __toESM,
  createStaticBuildAdapter,
  require_upath
} from "./chunk.RMFQVFYN.js";

// src/node/app/build/adapter/vercel/adapter.ts
var import_upath = __toESM(require_upath(), 1);
import esbuild from "esbuild";

// src/node/app/build/adapter/vercel/trailing-slash.ts
var trailingSlash = {
  keep: [
    {
      src: "^/(?:(.+)/)?index(?:\\.html)?/?$",
      headers: {
        Location: "/$1/"
      },
      status: 308
    },
    {
      src: "^/(.*)\\.html/?$",
      headers: {
        Location: "/$1/"
      },
      status: 308
    },
    {
      src: "^/\\.well-known(?:/.*)?$"
    },
    {
      src: "^/((?:[^/]+/)*[^/\\.]+)$",
      headers: {
        Location: "/$1/"
      },
      status: 308
    },
    {
      src: "^/((?:[^/]+/)*[^/]+\\.\\w+)/$",
      headers: {
        Location: "/$1"
      },
      status: 308
    }
  ],
  remove: [
    {
      src: "^/(?:(.+)/)?index(?:\\.html)?/?$",
      headers: {
        Location: "/$1"
      },
      status: 308
    },
    {
      src: "^/(.*)\\.html/?$",
      headers: {
        Location: "/$1"
      },
      status: 308
    },
    {
      src: "^/(.*)/$",
      headers: {
        Location: "/$1"
      },
      status: 308
    }
  ]
};

// src/node/app/build/adapter/vercel/adapter.ts
var outputRoot = ".vercel/output";
var defaultFunctionsConfig = {
  runtime: "nodejs16.x",
  handler: "index.js",
  maxDuration: 3,
  launcherType: "Nodejs",
  shouldAddHelpers: true,
  shouldAddSourcemapSupport: true
};
var defaultEdgeConfig = {
  runtime: "edge",
  entrypoint: "index.js"
};
var matchersRE = /\[(?:.*?)\]/g;
function createVercelBuildAdapter(config) {
  const isVercelEnv = !!process.env.VERCEL;
  return async (app, bundles, build, $) => {
    const vercelDirs = {
      root: $.createDirectory(app.dirs.root.resolve(`${outputRoot}`)),
      static: $.createDirectory(app.dirs.root.resolve(`${outputRoot}/static`)),
      fns: $.createDirectory(app.dirs.root.resolve(`${outputRoot}/functions`))
    };
    $.rimraf(vercelDirs.root.path);
    $.mkdirp(vercelDirs.root.path);
    const staticAdapter = await createStaticBuildAdapter({
      skipOutput: true,
      skipRedirects: isVercelEnv
    })(app, bundles, build, $);
    return {
      ...staticAdapter,
      name: "vercel",
      async write() {
        var _a;
        await ((_a = staticAdapter.write) == null ? void 0 : _a.call(staticAdapter));
        $.copyDir(app.dirs.client.path, vercelDirs.static.path);
        const redirects = Array.from(build.redirects.values()).map(
          (redirect) => ({
            src: redirect.from,
            headers: { Location: redirect.to },
            status: redirect.statusCode
          })
        );
        const overrides = {};
        for (const page of build.renders.values()) {
          overrides[page.filename] = {
            path: $.noendslash(page.filename.replace("index.html", ""))
          };
        }
        const routes = [
          ...(config == null ? void 0 : config.trailingSlash) ? trailingSlash.keep : trailingSlash.remove,
          ...redirects,
          {
            src: "/_immutable/.+",
            headers: { "Cache-Control": "public, immutable, max-age=31536000" }
          },
          { handle: "filesystem" }
        ];
        const bundlingFunctionsSpinner = $.createSpinner();
        const fnCount = $.color.underline(app.nodes.endpoints.size);
        bundlingFunctionsSpinner.start(
          $.color.bold(`Bundling ${fnCount} functions...`)
        );
        for (const endpoint of app.nodes.endpoints) {
          const apiPath = app.dirs.app.relative(endpoint.rootPath);
          const apiDir = import_upath.default.dirname(apiPath);
          routes.push({
            src: `^${$.slash(apiDir.replace(matchersRE, "([^/]+?)"))}/?$`,
            dest: $.slash(apiDir)
          });
        }
        const serverChunks = bundles.server.chunks;
        await Promise.all(
          Array.from(app.nodes.endpoints).map(async (endpoint) => {
            var _a2, _b;
            const chunk = serverChunks.find(
              (chunk2) => chunk2.facadeModuleId === endpoint.filePath
            );
            const allowedMethods = chunk == null ? void 0 : chunk.exports.filter(
              (id) => HTTP_METHODS.has(id)
            );
            if (!chunk || !allowedMethods || allowedMethods.length === 0) {
              return;
            }
            const isEdge = !!((_a2 = config == null ? void 0 : config.edge) == null ? void 0 : _a2.all) || chunk.exports.includes("EDGE");
            const code = resolveFunctionCode(
              endpoint.route.pathname,
              "./@http.js",
              allowedMethods
            );
            const vcConfig = isEdge ? {
              ...defaultEdgeConfig,
              envVarsInUse: (_b = config == null ? void 0 : config.edge) == null ? void 0 : _b.envVarsInUse
            } : {
              ...defaultFunctionsConfig,
              ...config == null ? void 0 : config.functions
            };
            const apiPath = app.dirs.app.relative(endpoint.rootPath);
            const fndir = `${import_upath.default.dirname(apiPath)}.func`;
            const outdir = vercelDirs.fns.resolve(fndir);
            const chunkdir = import_upath.default.dirname(
              app.dirs.server.resolve(chunk.fileName)
            );
            const entryPath = import_upath.default.resolve(chunkdir, "fn.js");
            await $.writeFile(entryPath, code);
            await esbuild.build({
              entryPoints: { index: entryPath },
              outdir,
              target: "es2020",
              assetNames: "assets/[name]-[hash]",
              chunkNames: "chunks/[name]-[hash]",
              bundle: true,
              splitting: true,
              minify: !app.config.isDebug,
              treeShaking: true,
              platform: "node",
              format: "esm",
              sourcemap: app.config.isDebug && "external"
            });
            await $.writeFile(
              import_upath.default.resolve(outdir, "package.json"),
              JSON.stringify({ type: "module" })
            );
            await $.writeFile(
              import_upath.default.resolve(outdir, ".vc-config.json"),
              JSON.stringify(vcConfig, null, 2)
            );
          })
        );
        bundlingFunctionsSpinner.stopAndPersist({
          text: $.color.bold(`Committed ${fnCount} functions`),
          symbol: $.icons.success
        });
        routes.push({
          src: "/(.*)",
          dest: "/index.html"
        });
        await $.writeFile(
          vercelDirs.root.resolve("config.json"),
          JSON.stringify({ version: 3, routes, overrides }, null, 2)
        );
      }
    };
  };
}
function resolveFunctionCode(pattern, moduleId, methods) {
  return [
    "import { createHTTPRequestHandler, installPolyfills } from 'vitebook/http';",
    "",
    "export default createHTTPRequestHandler(",
    `  () => new URLPattern({ pathname: '${pattern}' }),`,
    `  () => import('${moduleId}'),`,
    "  {",
    `    methods: [${methods.map((method) => `'${method}'`).join(", ")}],`,
    "    getBase: (req) => `https://${req.headers.host}`,",
    "    getClientAddress: (req) => req.headers['x-forwarded-for'],",
    "    installPolyfills,",
    "  }",
    ");",
    ""
  ].join("\n");
}
export {
  createVercelBuildAdapter,
  createVercelBuildAdapter as default
};
